package net.sf.nebulacards;

import java.net.*;
import java.io.*;

/**
 * Manage the communication threads for the server.
 * @author James Ranson
 */
public class ComManager
{
	private CommThread[] threads;
	private boolean[] resend;
	
	public ComManager( int n )
	{
		threads = new CommThread[n];
		resend = new boolean[n];
		for (int i = 0; i < n; i++)
		{
			threads[i] = null;
			resend[i] = false;
		}
	}

	/**
	 * Find a place for the given socket in the table.
	 * @param s A socket that is to be added to the table.
	 * @param b The initial state of the socket's resend flag.
	 * @return The position assigned to the socket, or -1 if table is full.
	 */
	public synchronized int add( Socket s, boolean b )
		throws IOException
	{
		int where = 0;
		while (where < threads.length && threads[where] != null)
			where++;
		if (where >= threads.length)
			return -1;
		threads[where] = new CommThread( s, where );
		resend[where] = b;
		return where;
	}

	/**
	 * Close a player's connection.
	 */
	public synchronized void remove( int i )
	{
		if (i >= 0 && i < threads.length && threads[i] != null)
		{
			threads[i].end();
			try { threads[i].join(); }
			catch (InterruptedException e) {}
			threads[i] = null;
		}
	}

	/**
	 * Get a comm thread.
	 */
	public synchronized CommThread getThread( int i )
	{
		return threads[i];
	}

	/**
	 * Send a packet to one player, only if that player's resend flag is true.
	 */
	public synchronized void resend( Object o, int i )
	{
		if (resend[i])
			send( o, i );
		resend[i] = false;
	}

	/**
	 * Send a packet to one player, only if that player's resend flag is false.
	 */
	public synchronized void noResend( Object o, int i )
	{
		if (!resend[i])
			send( o, i );
	}

	/**
	 * Send a packet to one player, and clear his resend flag.
	 * The prior state of the reset flag is irrelevant.
	 */
	public synchronized void sendClear( Object o, int i )
	{
		send( o, i );
		resend[i] = false;
	}

	/**
	 * Set a resend flag.
	 * @param b The new state of the resend flag.
	 * @param i The thread whose flag is to be set.
	 */
	public synchronized void setResend( boolean b, int i )
	{
		resend[i] = b;
	}

	/**
	 * Get a resend flag.
	 * @param i The index of the thread in question.
	 * @return The state of the resend flag.
	 */
	public synchronized boolean getResend( int i )
	{
		return resend[i];
	}


	/**
	 * Send a packet to one player.
	 */
	public synchronized void send( Object o, int i )
	{
		//System.err.println( "ComManager["+i+"] send: " + o );
		if (threads[i] != null)
			threads[i].send(o);
		//else System.err.println( "DEBUG: send to a null thread: " + i );
	}

	/**
	 * Broadcast to all players.
	 */
	public synchronized void broadcast( Object o )
	{
		for (int i = 0; i < threads.length; i++)
			if (threads[i] != null)
				threads[i].send( o );
	}

	/**
	 * Get number of active connections.
	 */
	public synchronized int howManyActive()
	{
		int count = 0;
		for (int i = 0; i < threads.length; i++)
			if (threads[i] != null)
				count++;
		return count;
	}
}
