package net.sf.nebulacards;
import java.net.*;
import java.io.*;
import java.util.*;
import net.sf.nebulacards.netpkt.*;

/**
 * A threaded class to handle the multiplexing of Nebula cards network traffic.
 * @author James Ranson
 */
public class CommThread
	extends Thread
{
	private ObjectInputStream in = null;
	private ObjectOutputStream out = null;
	private Socket mySocket = null;
	private Vector listeners = null;
	private boolean shouldStop = false;
	private int id;

	/**
	 * Construct a new multiplexer for the given socket.
	 *
	 * @param s A connected network socket.
	 * @param _id Some (unique) identification to report to packet listeners.
	 */
	public CommThread( Socket s, int _id )
		throws StreamCorruptedException, IOException
	{
		mySocket = s;
		mySocket.setSoTimeout( 0 );
		out = new ObjectOutputStream( mySocket.getOutputStream() );
		in = new ObjectInputStream( mySocket.getInputStream() );
		listeners = new Vector();
		id = _id;
		// unclosed sockets shouldn't hold up program exit.
		setDaemon( true );
	} 

	/**
	 * Sends data to the output channel.
	 * @param o An object to be output.
	 */
	public synchronized void send( Object o )
	{
		if (out == null) return;
		try
		{
			out.reset();
			out.writeObject( o );
			out.flush();
		}
		catch (IOException e) { end(); }
	}

	/**
	 * Specify an object that is to be informed of packet events.
	 * @param l Any PacketListener object.
	 */
	public void addPacketListener( PacketListener l )
	{
		if (l != null)
			listeners.addElement( l );
	}

	/**
	 * Take an object out of the notification list.
	 * @param l The listener to be removed.
	 */
	public void removePacketListener( PacketListener l )
	{
		listeners.removeElement( l );
	}

	/**
	 * This method runs when the thread is started and waits for socket input.
	 */
	public void run()
	{
		Object o;
		shouldStop = false;
		while (!shouldStop)
		{
			o = null;
			try { o = in.readObject(); }
			catch ( OptionalDataException e )
			{
				//System.err.println( "Exception in CommThread["+id+"]" );
				//e.printStackTrace( System.err );
				try { in.skip( in.available() ); }
				catch (Exception ee) {}
			}
			catch ( EOFException e )
			{
				//System.err.println( "CommThread["+id+"] reached EOF" );
				shouldStop = true;
			}
			catch ( Exception e )
			{
				//System.err.println( "CommThread[" + id + "]" );
				//e.printStackTrace( System.err );
			}
			finally
			{
				if (o != null) distribute( o );
			}
		}
		try { in.close(); } catch (Exception e) {}
		try { out.close(); } catch (Exception e) {}
		try { mySocket.close(); } catch (Exception e) {}
	}

	/** Send packet to listeners. */
	private void distribute( Object o )
	{
		for (int i = 0; i < listeners.size(); i++)
		{
			PacketListener pl = (PacketListener)listeners.elementAt(i);
			pl.packetReceived(id, o);
		}
	}

	/**
	 * Cleanly stop the execution of this thread.
	 * Stops thread and closes socket.
	 */
	public void end()
	{
		// closing the input stream should cause an exception which will
		// terminate the run method.
		shouldStop = true;
		try { in.close(); } catch (Exception e) {}
	}
}
