package net.sf.nebulacards.main;

/**
 * Used by a DirectCom rule set to tell the server to send a packet to a
 * client.  Packets marked as persistent are automatically resent when a
 * a player leaves and rejoins a game.  The list of persistent packets is
 * cleared at the end of each hand.
 * @see DirectComRules
 * @author James Ranson
 */
public class DirectComPacket
{
	private Object packet;
	private int who;
	private boolean persistent;
	
	/**
	 * Construct a packet.
	 * @param pkt The data to be sent in the packet.
	 * @param w Which player should receive the packet.
	 * @param p True if the packet is to be persistent, false otherwise.
	 */
	public DirectComPacket( Object pkt, int w, boolean p )
	{
		packet = pkt; who = w; persistent = p;
	}

	/**
	 * Construct a non-persistent packet.
	 * @param pkt The data to be sent in the packet.
	 * @param w Which player should receive the packet.
	 */
	public DirectComPacket( Object pkt, int w )
	{
		packet = pkt; who = w;
		persistent = false;
	}

	/**
	 * Construct an empty packet.
	 */
	public DirectComPacket()
	{
		packet = null;
		who = -1;
		persistent = false;
	}

	/**
	 * Get the packet data.
	 * @return The data to be sent to the client.
	 */
	public synchronized Object getData()
	{
		return packet;
	}

	/**
	 * Set the packet data.
	 * @param o The data to be sent to the client.
	 */
	public synchronized void setData( Object o )
	{
		packet = o;
	}

	/**
	 * Get the destination.
	 * @return Player who should receive the packet.
	 */
	public synchronized int getDestination() { return who; }

	/**
	 * Set the destination.
	 * @param w Player who should receive the packet.
	 */
	public synchronized void setDestination( int w ) { who = w; }

	/**
	 * Set persistence.
	 * @param p True if packet should be persistent, false otherwise.
	 */
	public synchronized void setPersistence( boolean p ) { persistent = p; }

	/**
	 * Is the packet persistent?
	 * @return True if packet is persistent, false otherwise.
	 */
	public synchronized boolean isPersistent() { return persistent; }
}
