package net.sf.nebulacards.main;

/**
 * This interface must be implemented by all rule sets that want to take
 * advantage of the direct communications kludge.  This allows rule sets to
 * implement rules that are not easily abstracted for placement in the game
 * engine.  The communication takes place after the built-in round of dealing,
 * bidding, and passing has taken place.  The rule set may send new cards
 * to any player, or ask any player to pass cards.
 * @author James Ranson
 */
public interface DirectComRules
{
	/**
	 * Indicate that the rule set is ready to send a communication.
	 */
	int READY = 0;
	/**
	 * Indicate that the rule set is waiting for a response from a player.
	 */
	int WAITING = 1;
	/**
	 * Indicate that the rule set has finished communicating with the players.
	 */
	int FINISHED = 2;

	/**
	 * Retrieve a packet to be sent to a player.  This is called as soon as
	 * the rule set indicates that it is READY.
	 * @see DirectComPacket
	 * @return The packet to be sent.
	 */
	DirectComPacket getPacket();

	/**
	 * Retrieve the current communication state of this rule set.
	 * @see #READY
	 * @see #WAITING
	 * @see #FINISHED
	 * @return One of the state constants (READY, WAITING, FINISHED).
	 */
	int getState();

	/**
	 * Advise the rule set of a packet that came in from the user.
	 * This method will not be called if getState() reports FINISHED.
	 * The following packet types are not forwarded to this method:
	 * <ul><li>Chat</li><li>Join</li><li>Leave</li></ul>
	 * @see #getState
	 * @param pkt The packet.
	 * @param who Who sent the packet.
	 */
	void packetReceived( Object pkt, int who );

	/**
	 * Advise the rule set that a user has disconnected and a new user
	 * has reconnected in his place.
	 * @param who The table position of the new player.
	 */
	void resend( int who );
}
