// July 1999

package net.sf.nebulacards.main;

/**
 * The set of functions that must be provided by all user interfaces
 * and computer players for Nebula Cards.<p>Table indexes
 * start at 0 for North, and go clockwise up to 3 for West.
 * <p>Implementations <b>must</b> provide a constructor.  This constructor
 * can take no parameters, or it can take a single <b>int</b> parameter that
 * will be the table index where the player is sitting.
 *
 * @author James Ranson
 */
public interface NebulaUi
{
    /**
     * Table index
     */
    int NORTH = 0;

    /**
     * Table index
     */
    int EAST = 1;

    /**
     * Table index
     */
    int SOUTH = 2;

    /**
     * Table index
     */
    int WEST = 3;

    /**
     * Set the trump suit to the given suit.
     *
     * @param t A suit constant from PlayingCard
	 * @param n A text representation of the trump suit.
     */
    void setTrump( int t, String n );

	/**
	 * Answer a generic query.  These queries come from the rule set using the
	 * DirectComm hack.  The question is posed in natural language,
	 * and the interface is expected to return some string in reply.  However,
	 * to ease the burden on robot interfaces, we specify that an empty string
	 * is considered a "no response" or "don't understand" signal, and
	 * individual rule sets may handle it as they please.
	 * @param query The question being asked.
	 * @return The user's response.
	 */
	String respond( String query );

	/**
	 * Set the game name.
	 *
	 * @param n The name of the game as provided by the server's rule set.
	 */
	void setGameName( String n );

    /**
     * Set the bid for a specific player.
     * Bids are broadcast as soon as they are entered, so you can see the
     * bids of those who bid before you.
     *
     * @param who The table index of the player whose bid is being sent.
     * @param bid The amount bid.
     */
    void setBid( int who, int bid );

	/**
	 * Update information for all the players.
	 *
	 * @param p The players.
	 */
	void setPlayers( Player[] p );

    /**
     * Called when someone plays a card to the tableau.
     *
     * @param pos The table index of the player who played this card.
     * @param c The card that was played.
     */
    void cardToTableau( int pos, PlayingCard c );

    /**
     * Called after the trick is complete.
     *
     * @param whoWonTrick The table index of the player who won the trick.
     */
    void clearTableau( int whoWonTrick );

    /**
     * Called when you are dealt cards.
     *
     * @param h Your hand of cards.
     */
    void dealHand( PileOfCards h );

    /**
     * Called to retrieve the card you want to play.
     * Will only be called when your object's ready() method returns true.
     *
     * @return The card you want to play.
     */
    PlayingCard getPlay();
    
    /**
     * Called to retrieve the bid you want to make.
     * Will only be called when your object's ready() method returns true.
     *
     * @return The bid you want to make.
     */
    int getBid();


    /**
     * Called to retrieve the cards you want to pass.
     * Will only be called when your object's ready() method returns true.
     *
     * @return The cards you want to pass.
     */
    PileOfCards getPass();


    /**
     * Informs you that it is your turn to play a card.
     * The card will be retrieved by a later call to getPlay()
     *
     * @see #getPlay(void)
     */
    void yourTurn();

    /**
     * Informs you that it is your turn to bid.
     * The bid will be retrieved by a later call to getBid()
     *
     * @see #getBid(void)
     */
    void yourTurnToBid();


    /**
     * Informs you that it is your turn to pass.
     * The cards will be retrieved by a later call to getPass() once your
	 * ready() method returns true.
     *
	 * @param howmany How many cards you must pass.
	 * @param who The position number of the player who receive the cards.
     * @see #getPass(void)
     */
    void yourTurnToPass( int howmany, int who );


    /**
     * Called periodically to see if you are ready to play or bid.
     *
     * @return True if your getBid() or getPlay() method is ready to be
     * called, false otherwise.
     */
    boolean ready();

    /**
     * Called to signal acceptance of the last card you played.
     * This is only called for cards, not for bids.
     *
     * @see #rejected(void)
     */
    void accepted();

    /**
     * Called to signal rejection of the last card you played.
     * This is only called for cards, not for bids.
     *
     * @see #accepted(void)
     */
    void rejected();

    /**
     * Chat message receiver.
     * Chat messages can come from other players or they may be messages
     * from the server.
     *
     * @param s The message.
     */
    void chat( String s );

    /**
     * Called to see if you want to send a message.
     *
     * @return True if you want to send a message, false otherwise.
     */
    boolean chatReady();

    /**
     * Called to retrieve the message you want to send.
     * The server automatically adds your name to the beginning of the
     * message before it is broadcast.
     *
     * @return The message.
     */
    String getChat();

    /**
     * Called when the game is over.
     * Your class must use this method to close all its windows and tidy up,
     * because no other calls will be made after this.
     *
     * @param gr The result of the game.
     */
    void endGame( GameResult gr );

    /**
     * Called to see if you want to quit.
     * If this returns true, the client program will tell the server it is
     * leaving and then terminate.  No further calls will be made to your
     * class once this method returns true.
     *
     * @return True if you want to quit, false otherwise.
     */
    boolean wantToQuit();

	/**
	 * Called when the hand is over.
	 * This method is called when all players have played out all their cards.
	 */
	void endHand();

    /**
     * Called when you are booted from the game
     *
     * @param why Explains why you were booted.
     */
    void booted( String why );

	/**
	 * Called when you enter a game that is already in progress.
	 * This method is called to inform your interface of all cards that were
	 * played before you joined the game.  Note that the cards in the
	 * current trick are sent regardless of this interface.  This is only
	 * for cards played on previous tricks.
     *
     * @param bp The cards that have been played so far in this hand.
     */
    void playedSoFar( PileOfCards bp );

	/**
	 * Called when the client receives a communication that doesn't match
	 * any of the standard types.  This method is used only by DirectCom rule
	 * sets, so unless your client is expecting non-standard information from
	 * the rule set, it is safe to ignore the data passed by this method.
	 *
	 * @param o The data passed by the rule set.
	 */
	void miscInfo( Object o );
}
