// July 1999
package net.sf.nebulacards.main;

import java.util.*;
import java.io.*;

/**
 * Container class for a pile of playing cards.
 *
 * @author James Ranson
 * @see PlayingCard
 */
public class PileOfCards
    implements Serializable
{
    /**
     * Returned by search methods if query not found.
     */
    public static final int NOT_FOUND = -1;

    // private data, no docs
    protected Vector pile;

    /**
     * Constructs an empty pile.
     */
    public PileOfCards()
    {
        pile = new Vector();
    }

    /**
     * Constructs an empty pile
     *
     * @param size Initial capacity of card vector.
     */
    public PileOfCards( int size )
    {
        pile = new Vector(size);
    }

	/**
	 * Returns a standard 52-card deck.
	 * @return A pile of cards containing all 13 cards in each of the 4 suits.
	 */
	public static PileOfCards getStandardDeck()
	{
        PileOfCards p = new PileOfCards(52);
        for (int suit = 0; suit < 4; suit++)
            for (int v = 1; v <= 13; v++)
                p.addToTop( new PlayingCard(v, suit) );
		return p;
 	}

    /**
     * Returns the number of cards actually stored in this pile.
     */
    public synchronized int howManyCards() { return pile.size(); }

    /**
     * Retrieve any card from the pile.
     * Does not remove the card from the pile.
     *
     * @see #takeCard(int)
     * @param index The index of the card, where 0 is the bottom of the pile.
     * @return The requested card.
     */
    public synchronized PlayingCard getCard( int index )
	{
		return (PlayingCard)pile.elementAt(index);
	}

    /**
     * Retrieve the top card.
     * Does not remove the card from the pile.
     *
     * @see #takeTop()
     * @return The top card.
     */
    public synchronized PlayingCard getTop()
	{
		return (PlayingCard)pile.lastElement();
	}

    /**
     * Remove the top card.
     *
     * @see #getTop()
     * @return The top card.
     */
    public synchronized PlayingCard takeTop()
    {
        PlayingCard t = (PlayingCard)pile.lastElement();
		pile.removeElementAt( pile.size() - 1 );
        return t;
    }

    /**
     * Remove any card.
     *
     * @see #getCard(int)
     * @param index The index of the card, where 0 is the bottom.
     * @return The card that was removed.
     */
    public synchronized PlayingCard takeCard( int index )
    {
        PlayingCard t = (PlayingCard)pile.elementAt(index);
		pile.removeElementAt(index);
        return t;
    }

    /**
     * Adds a card to the top of the pile.
     *
     * @see #addCard(PlayingCard,int)
     * @param c The card to be added to the top.
     */
    public synchronized void addToTop( PlayingCard c )
    {
		pile.addElement( c );
    }

    /**
     * Adds another pile of cards to the top of this pile.
     * The order of the added pile is preserved.
     *
     * @param c The pile of cards to be added to the top of this pile.
     */
    public synchronized void addToTop( PileOfCards c )
    {
		pile.ensureCapacity( c.howManyCards() + this.howManyCards() );
        for (int i = 0; i < c.howManyCards(); i++)
            addToTop( c.getCard(i) );
    }

    /**
     * Inserts a card at the given index.
     * Does <b>not</b> overwrite the card already there.
     *
     * @param c The card to be inserted.
     * @param index The index where the new card will reside.
     */
    public synchronized void addCard( PlayingCard c, int index )
    {
		pile.insertElementAt( c, index );
    }

    /**
     * Randomizes the order of the cards.
     */
    public synchronized void shuffle()
    {
        if ( howManyCards() < 2 ) return;
        
        PlayingCard tmp;
        Random r = new Random();

        for (int i = 0; i < howManyCards(); i++) {
            int j = Math.abs(r.nextInt()) % howManyCards();
            tmp = (PlayingCard)pile.elementAt(j);
            pile.setElementAt( pile.elementAt(i), j );
            pile.setElementAt( tmp, i );
        }
    }

    /**
     * Searches for the given card in this pile.
     *
     * @see #search(java.lang.String)
     * @see #NOT_FOUND
     * @param c The card to be found.
     * @return The index of the card or NOT_FOUND.
     */
    public synchronized int search( PlayingCard c )
    {
        for (int i = 0; i < howManyCards(); i++)
            if (getCard(i).equals( c )) 
				return i;
		return NOT_FOUND;
    }

    /**
     * Searches for the given string in this pile.
     * Finds a card whose toString() matches the given string.
     *
     * @see #search(PlayingCard)
     * @see #NOT_FOUND
     * @see PlayingCard#toString()
     * @param s The string to be matched.
     * @return The index of the card or NOT_FOUND
     */
    public synchronized int search( String s )
    {
        for (int i = 0; i < this.howManyCards(); i++)
            if (getCard(i).toString().equals( s ))
                return i;
        return NOT_FOUND;
    }

    /**
     * Does the pile contain any cards of the given suit?
     *
     * @see PlayingCard
     * @param suit Any suit constant from PlayingCard
     * @return True if that suit exists, false otherwise.
     */
    public synchronized boolean anyOfThisSuit( int suit )
    {
        return (cardsInSuit( suit ).howManyCards() > 0);
    }

    /**
     * Do all the cards in the pile have the given suit?
     *
     * @param suit Any suit constant from PlayingCard
     */
    public synchronized boolean allThisSuit( int suit )
    {
        return (cardsInSuit( suit ).howManyCards() == howManyCards());
    }

    /**
     * Retrieve cards that have the given suit.
     * Does <b>not</b> remove cards from pile.
     *
     * @param suit Any suit constant from PlayingCard.
     * @return All the cards in this pile that have the given suit.
     */
    public synchronized PileOfCards cardsInSuit( int suit )
    {
        PileOfCards pc = new PileOfCards( howManyCards() );
        for (int i = 0; i < howManyCards(); i++)
            if ( getCard( i ).getSuit() == suit )
                pc.addToTop( getCard(i) );
        return pc;
    }

	
    /**
     * Remove all instances of the given card.
     *
	 * @deprecated Use remove(PlayingCard)
	 * @see #remove(PlayingCard)
     * @see #search(PlayingCard)
     * @param c The card to be removed from the pile.
     */
	public synchronized void removeCard( PlayingCard c ) { remove(c); }


    /**
     * Remove all instances of the given card.
     *
     * @see #search(PlayingCard)
     * @param c The card to be removed from the pile.
     */
    public synchronized void remove( PlayingCard c )
    {
		int i = 0;
		while ( i < howManyCards() )
		{
			if (getCard(i).equals(c))
				takeCard(i);
			else i++;
		}
    }

    /**
     * Remove all the cards from this pile that are found in the given pile.
     *
	 * @deprecated
	 * @see #remove(PileOfCards)
     * @param p The cards that should be removed from this pile.
     */
	public synchronized void removePile( PileOfCards p ) { remove(p); }

    /**
     * Remove all the cards from this pile that are found in the given pile.
     *
     * @param p The cards that should be removed from this pile.
     */
    public synchronized void remove( PileOfCards p )
    {
        for (int i = 0; i < p.howManyCards(); i++)
            removeCard( p.getCard(i) );
    }

    /**
     * Remove all cards of the given suit.
     *
     * @param suit The suit of the cards to be removed.
     */
    public synchronized void removeSuit( int suit )
    {
        for (int i = 0; i < howManyCards(); i++)
            if (getCard(i).getSuit() == suit)
                takeCard( i );
    }

	/**
	 * Remove all cards from the pile.
	 */
	public synchronized void clear()
	{
		pile.removeAllElements();
	}

    /**
     * Convert this pile of cards to a String.
     * Concatenates the string representations of each card.
     * Creates a 13-column table.
     *
     * @return String representation of this pile.
     */
    public synchronized String toString()
    {
        String s = "";
        int index = 0;
        for (int i = 0; i < howManyCards() / 13; i++) {
            for (int j = 0; j < 13; j++, index++)
                s += pile.elementAt(index).toString() + " ";
            s += "\n";
        }
        for (; index < howManyCards(); index++)
            s += pile.elementAt(index).toString() + " ";
        return s;
    }

    /**
     * Sort all cards by suit, and by pip count within each suit.
     * Aces are high.
     */
    public synchronized void sort()
    {
		// do a bubble sort
		int numCards = howManyCards();
		for (int i = 1; i < numCards; i++)
		{
			for (int j = 0; j < numCards - i; j++)
			{
				PlayingCard a = getCard(j), b = getCard(j+1);
				boolean swap = false;
				if (a.getSuit() > b.getSuit()) 
					swap = true;
				else if (a.getSuit() == b.getSuit()
						&& a.getValue() > b.getValue())
					swap = true;
				if (swap)
				{
					pile.setElementAt( b, j );
					pile.setElementAt( a, j + 1 );
				}
			}
		}
    }

    /**
     * Constructs a copy of the given pile.
     *
     * @return A copy of the given pile.
     */
    public PileOfCards( PileOfCards p )
    {
        pile = new Vector( p.pile.size() );
        for (int i = 0; i < p.howManyCards(); i++)
            pile.addElement( p.getCard( i ).copy() );
    }

	/**
	 * Return an enumeration of the cards in this pile.
	 * @return An enumeration of the card vector.
	 */
	public Enumeration elements() { return pile.elements(); }
}
