// A single playing card
// James Ranson
// July 1999

package net.sf.nebulacards.main;

import java.io.Serializable;

/**
 * A single playing card.
 * Pip count is 1-based. (i.e. ace=1, 11=jack, 12=queen, ...)
 *
 * @author James Ranson
 */
public class PlayingCard
    implements Cloneable, Serializable
{
    /**
     * Card is a spade.
     */
    public static final int SPADES = 0;
    /**
     * Card is a diamond.
     */
    public static final int DIAMONDS = 1;
    /**
     * Card is a club.
     */
    public static final int CLUBS = 2;
    /**
     * Card is a heart.
     */
    public static final int HEARTS = 3;
    
    // data items
    protected int suit, val;  // val is pip count ( 1 to 13 )


    /**
     * Make a new playing card.
     */
    public PlayingCard() { suit = SPADES; val = 1; }
    
    /**
     * Make a new playing card with the given suit and value.
     *
     * @param v The value (i.e. pip count)
     * @param s The suit (use one of the suit constants)
     */
    public PlayingCard( int v, int s ) { suit = s; val = v; }

	/**
	 * Copy constructor.
	 */
	public PlayingCard( PlayingCard c )
	{
		suit = c.getSuit();
		val = c.getVal();
	}

    /**
     * The suit of this card.
     *
     * @return The suit.
     */
    public synchronized int getSuit() { return suit; }
    
    /**
     * The value of this card.
     *
     * @return The value.
     */
    public synchronized int getVal() { return val; }

    /**
     * The value of this card.
     * Same as getVal().
     *
     * @see #getVal
     * @return The value.
     */
    public synchronized int getValue() { return val; }
    public synchronized void setValue( int v ) { val = v; }
    public synchronized void setSuit( int s ) { suit = s; }
    public synchronized PlayingCard copy()
	{ return new PlayingCard( this.getValue(), this.getSuit() ); }

    /**
     * Is <b>this</b> card higher than the given card?
     *
     * @param c The card to be checked against this card.
     * @param AceHigh Should aces be considered higher than kings.
     * @return True if this card is higher, false otherwise.
     */
    public synchronized boolean higherThan( PlayingCard c, boolean AceHigh )
    {
        if (AceHigh) {
            if (val == 1 && c.getValue() != 1) return true;
            if (c.getValue() == 1) return false; // can't be higher than an ace
            // neither card is an ace, so just do a pip comparison
            return (val > c.getValue());
        }
        else  // if aces are low, than just do a pip comparison
            return (val > c.getValue());
    }

	/**
	 * Is <b>this</b> card is exactly the same as the given card?
	 *
	 * @param c The card to be checked against this card.
	 * @return True if cards are the same, false otherwise.
	 */
	public synchronized boolean equals( PlayingCard c )
	{
		return (c.suit == this.suit && c.val == this.val);
	}

    /**
     * Convert this card to a string.
     * Consists of the value (as a number or capital letter), followed by the
     * first letter of the suit.
     *
     * @return The String representation of this card.
     */
    public synchronized String toString()
    {
        String s;
        switch( val )
        {
        case 1: s = "A"; break;
        case 11: s = "J"; break;
        case 12: s = "Q"; break;
        case 13: s = "K"; break;
        default: s = String.valueOf( (int)val ); break;
        }
        switch( suit )
        {
        case SPADES: s += "s"; break;
        case HEARTS: s += "h"; break;
        case CLUBS: s += "c"; break;
        case DIAMONDS: s += "d"; break;
        }
        return s;
    }

    /**
     * Make a card from a string representation.
     * Reverses the process of toString()
     *
     * @param s The string representation of the card.
     * @return A new card.
     * @see #toString(void)
     */
    public static PlayingCard fromString( String s )
    {
        int val, suit;
        switch( s.charAt(0) )
        {
        case 'A': val = 1; break;
        case '1': val = 10; break;
        case 'J': val = 11; break;
        case 'Q': val = 12; break;
        case 'K': val = 13; break;
        default: val = (s.charAt(0) - '0');
        }
        switch( s.charAt( s.length() - 1) )
        {
        case 's': suit = SPADES; break;
        case 'd': suit = DIAMONDS; break;
        case 'c': suit = CLUBS; break;
        case 'h': suit = HEARTS; break;
        default: return null;
        }
        return new PlayingCard( val, suit );
    }
}
