// July 1999

package net.sf.nebulacards.main;
import java.util.Enumeration;

/**
 * Base class for card game rules.
 *
 * @author James Ranson
 */
public abstract class Rules
{
    /**
     * Indicates that no trump suit is set.
     *
     * @see #trump
     */
    public static final int NO_TRUMP = -1;

    /**
     * The trump suit.  A suit constant from PlayingCard or NO_TRUMP.
     * This MUST be set to a valid value if you want to use the default
     * whoWinsTrick method.
     *
     * @see #NO_TRUMP
     * @see nebula.core.PlayingCard
     */
    protected int trump;

	/**
	 * A text name for the trump suit.  This helps user interfaces to display
	 * something meaningful, instead of just a number.
	 */
	protected String trumpName;

	/**
	 * Deal the cards.  This method is called to distribute cards to the
	 * players.  The default implementation deals out all the cards in a
	 * standard deck.
	 * @return The cards in each player's hand.
	 */
	public PileOfCards[] deal()
	{
		PileOfCards deck = PileOfCards.getStandardDeck();
		deck.shuffle();
		PileOfCards[] hands = new PileOfCards[4];
		for (int i = 0; i < 4; i++)
			hands[i] = new PileOfCards( 13 );
		Enumeration cards = deck.elements();
		int who = 0;
		while (cards.hasMoreElements())
		{
			hands[who].addToTop( (PlayingCard)(cards.nextElement()) );
			who++; who %= 4;
		}
		return hands;
	}

    /**
     * Identifies this set of rules.
     * The name provided by this method is passed to the interfaces when
     * a game commences using this rule set.
     *
     * @return The name of the game.
     */
    public abstract String name();
    
    /**
     * Get the trump suit.  This returns the value stored in the <i>trump</i>
	 * member variable.
     *
	 * @see #trump
     * @return The trump suit.
     */
    public final int getTrump() { return trump; }

    /**
     * Get the trump suit name.  This returns the value stored in the
	 * <i>trumpName</i> member variable.
     *
	 * @see #trumpName
     * @return The trump suit name.
     */
    public final String getTrumpName() { return trumpName; }

    /**
     * Set the trump suit.  The default behaviour is to ignore the card
     * and set the value to NO_TRUMP.
     *
     * @see #NO_TRUMP
     * @param c The final card dealt.
     */
    public void setTrump( PlayingCard c ) { trump = NO_TRUMP; }

    /**
     * Check if the given card is a legal play in this context.
     * This is entirely dependent on the particular game, so no default behaviour
     * is provided.
     *
     * @param beenPlayed All the cards that have been played during this hand.
     * @param tableau The cards that are currently on the table.
     * @param hand The players hand, including the card in question.
     * @param c The card to be verified.
     * @return True if card is a legal play, false otherwise.
     */
    public abstract boolean validPlay( PileOfCards beenPlayed,
                              PileOfCards tableau,
                              PileOfCards hand,
                              PlayingCard c );

    /**
     * Determine who wins the trick.
	 * This method must NOT assume that the table is full.
	 *
     * DEFAULT BEHAVIOUR:
     *  Highest-card-wins, including trump cards if any, ace high
     *
     * @param tableau All the cards in the trick.
     * @return The table position of the winning card.
     */
    public int whoWinsTrick( Tableau tableau )
    {
        PlayingCard high = tableau.getTableCard( tableau.getLead() );
        int winner = tableau.getLead();

		if (high == null)
			return winner;

	    for (int i = 0; i < 4; i++)
		{
			if (i == tableau.getLead())
				continue;
   	    	PlayingCard c = tableau.getTableCard( i );
			if (c == null)
			{
				continue;
			}
	        else if (c.getSuit() == trump && high.getSuit() != trump) 
			{
   				high = c;
               	winner = i;
           	}
           	else if (c.getSuit() == high.getSuit() && c.higherThan(high, true))
			{
               	high = c;
               	winner = i;
        	}
        }
        return winner;
    }


	/**
	 * Update the scores and bags.
	 * Calculate and set the new scores and bags.
	 * By default, this calls score( PileOfCards[], int[], int[], int[] ).
	 *
	 * @param hands The cards each player won from tricks.
	 * @param p The players.
	 */
	abstract public void score( PileOfCards[] hands, Player[] p );


	/**
	 * Provide a result for this game.
	 * Called after every hand to see if the game is done.
	 *
	 * @param p The players.
	 * @return The result of the game.
	 */
	abstract public GameResult done( Player[] p );
}

