package net.sf.nebulacards.main;

/**
 * A tableau (that is, the cards that have been played in the current trick).
 * This is an extension of PileOfCards that makes it easier to reference
 * cards by their table position, not just the order in which they were
 * played.  It does this by keeping track of who led (i.e. who played
 * card 0).
 * @author James Ranson
 */
public class Tableau
	extends PileOfCards
{
	private int whoLed;
	private int numPlayers;

	/**
	 * Constructor
	 * @param n Number of places at the table.
	 */
	public Tableau( int n )
	{
		super( n );
		numPlayers = n;
		whoLed = -n;
	}

	/**
	 * Default constructor (assumes 4 places).
	 */
	public Tableau()
	{
		super( 4 );
		numPlayers = 4;
		whoLed = -4;
	}

	/**
	 * Set leader.
	 * @param n The table position of the lead card.
	 */
	public synchronized void setLead( int n ) { whoLed = n; }

	/**
	 * Get leader.
	 * @return The table position of the lead card.
	 */
	public synchronized int getLead() { return whoLed; }

	/**
	 * Get the card at a specified table position.
	 * @param n The table position (valid range is 0 to numPlayers-1).
	 * @return The card at the position, or null if there is no card there.
	 */
	public synchronized PlayingCard getTableCard( int n )
	{
		PlayingCard c;
		try {
			c = super.getCard( pileIndex(n) );
		} catch (ArrayIndexOutOfBoundsException e) {
			c = null;
		}
		return c;
	}

	/**
	 * Does a given spot have a card in it?
	 * @param n A table index.
	 * @return true if a card is there, false otherwise
	 */
	public synchronized boolean hasPlayed( int n )
	{
		return (getTableCard(n) != null);
	}

	/**
	 * Return the pile index that corresponds to a certain table position.
	 * @param n The table position.
	 * @return The corresponding index in the underlying vector.
	 */
	private synchronized int pileIndex( int n )
	{
		if (n >= whoLed)
			return n - whoLed;
		else
			return n + numPlayers - whoLed;
	}

	/**
	 * Convert the tableau to a string.
	 * Table position 0 is listed first.  Empty spots are denoted by
	 * <b>empty</b>.
	 */
	public synchronized String toString()
	{
		String s = "";
		for (int i = 0; i < numPlayers; i++)
		{
			PlayingCard c = getTableCard(i);
			if (c == null)
				s += "empty ";
			else
				s += c + " ";
		}
		return s;
	}
}
