package net.sf.nebulacards.uicomponents;
import net.sf.nebulacards.main.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Simple, button-based card selector.
 * @author James Ranson
 */
public class BasicCardSelector
	extends Panel
	implements ActionListener, CardSelector
{
	protected PlayingCard[] hand;
	protected Vector listeners;
	protected Button[] buttons;
	protected int numButtons;
	protected GridLayout layout;

	/**
	 * Construct a selector with the given number of blank buttons.
	 * @param num The number of blank buttons.
	 */
	public BasicCardSelector( int num )	{ construct( num ); }

	/**
	 * Construct a selector with the default number of blank buttons.
	 */
	public BasicCardSelector() { construct( 1 ); }

	/**
	 * Common subset of the constructor methods.
	 * @param num The number of buttons to have initially
	 */
	private void construct( int num )
	{
		buttons = null;
		listeners = new Vector( 2 );
		hand = null;
		layout = new GridLayout( 0, 1 );
		setLayout( layout );
		numButtons = 0;
		resize( num );
	}

	/**
	 * Change the number of buttons.
	 * @param num The desired number of buttons.
	 */
	protected synchronized void resize( int num )
	{
		// avoid resizing if possible
		if (num == numButtons)
			return;
		// get rid of extra buttons
		for (int i = num; i < numButtons; i++)
		{
			buttons[i].removeActionListener( this );
			remove( buttons[i] );
			buttons[i] = null;
			hand[i] = null;
		}

		// make sure array exists!
		if (buttons == null)
		{
			buttons = new Button[num];
			hand = new PlayingCard[num];
		}
		// extend array if necessary
		if (num > buttons.length)
		{
			Button[] btemp = buttons;
			PlayingCard[] htemp = hand;
			buttons = new Button[num];
			hand = new PlayingCard[num];
			for (int i = 0; i < btemp.length; i++)
			{
				buttons[i] = btemp[i];
				hand[i] = htemp[i];
			}
		}
		layout.setColumns( num );
		// add new buttons
		for (int i = numButtons; i < num; i++)
		{
			buttons[i] = new Button( "___" );
			buttons[i].addActionListener( this );
			hand[i] = null;
			add( buttons[i] );
		}
		numButtons = num;
		layout.layoutContainer( this );
	}
			
	/**
	 * Set the labels on all the card buttons.  Sets empty buttons to blank.
	 * @param h The new hand.
	 */
	protected synchronized void setLabels( PileOfCards h )
	{
		Enumeration e = h.elements();
		for (int i = 0; i < numButtons; i++)
		{
			if (e.hasMoreElements())
			{
				PlayingCard n = (PlayingCard)(e.nextElement());
				buttons[i].setLabel( n.toString() );
				hand[i] = n;
			}
			else 
			{
				buttons[i].setLabel( "" );
				hand[i] = null;
			}
		}
	}

	/**
	 * Set the cards that are available in this selector.
	 * @param p The new pile of cards.
	 */
	public void setHand( PileOfCards p )
	{
		resize( p.howManyCards() );
		setLabels( p );
	}

	/**
	 * Get the cards that are available in this selector.
	 * @return A duplicate of the pile of cards available in this selector.
	 */
	public PileOfCards getHand() { return (PileOfCards)(hand.clone()); }
	
	/**
	 * Remove a card from this selector, if possible.
	 * @param c The card to remove.
	 */
	public void removeCard( PlayingCard c )
	{
		// search for card among button labels, then blank it.
		for (int i = 0; i < numButtons; i++)
		{
			if (hand[i] != null && hand[i].equals( c ))
			{
				buttons[i].setLabel( "" );
				hand[i] = null;
				break;
			}
		}
	}

	/**
	 * Add an object to this object's notify list.  All objects in the
	 * notify list will have their <i>cardSelected</i> method called when
	 * a card is selected.
	 * @param o The listener.
	 */
	public void addListener( CardListener o ) 
	{
		listeners.addElement( o ); 
	}

	/**
	 * Remove an object from this object's notify list.
	 * @param o The listener.
	 */
	public void removeListener( CardListener o )
	{
		listeners.removeElement( o );
	}
	
	/**
	* Listen for button pushes. Calls the listeners when an event occurs.
	* @param e The action event representing the button push.
	*/
	public void actionPerformed( ActionEvent e )
	{
		PlayingCard c = null;
		Object src = e.getSource();
		if (src instanceof Button)
		{
			for (int i = 0; i < numButtons; i++)
			{
				if (src == buttons[i])
				{
					c = hand[i];
					//System.out.println( "Selected card: " + c );
					break;
				}
			}
			if (c != null)
			{
				Enumeration elems = listeners.elements();
				while (elems.hasMoreElements())
					((CardListener)(elems.nextElement())).cardSelected( c );
			}
		}
	}
}
