package net.sf.nebulacards.uicomponents;

import net.sf.nebulacards.main.*;
import net.sf.nebulacards.rules.*;

/**
 * Stub class for computer players that keeps track of the game so that
 * derived classes need only worry about thinking.
 * @author James Ranson
 */
public abstract class ComputerStub
    implements NebulaUi
{
    private boolean verbose = false;
	protected String gameName = null;
    protected int whereAmI;
	protected int trump;
    protected int[] scores = new int[4], bags = new int[4];
    protected int[] bids = new int[4], tricks = new int[4];
    protected boolean[] hasBid = new boolean[4];
    protected PileOfCards hand = null, beenPlayed;
	protected Tableau tableau;
	protected boolean wantToQuit = false;
    
    // we'll keep track of the cards that COULD be in the other player's hands
    protected PileOfCards[] otherHands = new PileOfCards[4];
    
    // constructor
    public ComputerStub( int w )
    {
        whereAmI = w;

        for (int i = 0; i < 4; i++) {
            scores[i] = 0;
            bags[i] = 0;
            bids[i] = 0;
            hasBid[i] = false;
            tricks[i] = 0;
        }
        tableau = new Tableau( 4 );
        beenPlayed = new PileOfCards();
    }


    // we need to know what has gone on before we joined the game
    public void playedSoFar( PileOfCards bp )
    {
        beenPlayed = bp;
        if (verbose) {
            System.out.println( "Past tricks:" );
            System.out.println( beenPlayed );
        }
    }

    
    // we need to keep track of these items
	public void setPlayers( Player[] p )
	{
		scores = new int[p.length]; bags = new int[p.length];
		for (int i = 0; i < p.length; i++)
		{
			scores[i] = p[i].getScore();
			bags[i] = p[i].getBags();
			if (verbose) System.out.println( p[i] );
		}
	}

    public void endHand()
    {
        if (verbose) {
            System.out.println( "Score/Bags:" );
            for (int i = 0; i < 4; i++)
                System.out.println( scores[i] + "/" + bags[i] );
        }
    }
    
    public void setBid( int who, int bid )
    {
        if (verbose)
            System.out.println( "#" + who + " bids " + bid );
        bids[who] = bid;
        hasBid[who] = true;
    }
    
    public void dealHand( PileOfCards h )
    {
        hand = h;
        for (int i = 0; i < 4; i++) {
            hasBid[i] = false;
            tricks[i] = 0;
            // reset possible hands
            if (i != whereAmI) {
                otherHands[i] = PileOfCards.getStandardDeck();
                // they can't have cards that we have
                for (int j = 0; j < hand.howManyCards(); j++)
                    otherHands[i].remove( hand.getCard(j) );
            }
        }
        beenPlayed = new PileOfCards();
    }
    
    public void cardToTableau( int pos, PlayingCard c )
    {
        if (tableau.howManyCards() == 0)
			tableau.setLead( pos );
        tableau.addToTop( c );
        beenPlayed.addToTop( c );
        
        // card has been played, so nobody can have it in his hand
        for (int i = 0; i < 4; i++)
            if (i != whereAmI)
                otherHands[i].remove( c );
    }
    
    public void clearTableau( int whoWonTrick )
    {
        tricks[whoWonTrick]++;
        tableau.clear();
    }

    public void setTrump( int t, String n )
	{
		trump = t;
	}

	public void setGameName( String n )
	{
		gameName = n;
	}

	// assume computer players don't care about chat.
    public void chat( String s ) {}
    public boolean chatReady() { return false; }
    public String getChat() { return null; }

	public void booted( String why )
	{
		if (verbose)
			System.out.println( "Booted because: " + why );
	}

    public void endGame( GameResult gr )
    {
        if (verbose) System.out.println( gr );
		wantToQuit = true;
    }
    public boolean wantToQuit() { return wantToQuit; }
    
    public void yourTurnToBid() { if (verbose) System.out.println( "My turn to bid" ); }
    public void yourTurn() { if (verbose) System.out.println( "My turn to play." ); }
	public void yourTurnToPass(int howmany, int who ) {}

    public boolean ready() { return true; }

	protected abstract PlayingCard thinkPlay();

	public PlayingCard getPlay()
	{
		PlayingCard c = thinkPlay();
		hand.remove(c);
		return c;
	}

	/**
	 * Ignore misc info by default.
	 */
	public void miscInfo( Object o ) {}
}
